#!/usr/local/bin/perl -d
#
#
# This will filter MARC records from the zclient and add HTML formatting.
#
# Author: Harold Finkbeiner (harold@lindy.stanford.edu)
#

$HREF = "http://$ENV{SERVER_NAME}:$ENV{SERVER_PORT}$ENV{SCRIPT_NAME}?";
$BREF = '<a href="' . $HREF;	# Matching quote (") for perl formatting.
$PREF = $BREF . $ZDB;

if (defined($FORM{marc})) { &marc_display(); }
elsif ($elemname eq 'F')  { &full_display(); }
else                      { &brief_display(); }

sub full_display {
    &setup_hypertext();

    print '<DL>';

    while (<STDIN>) {
	s/\d+/$FORM{disnum}/ if (defined($FORM{disnum}));
	print "<DT><H3>$_</H3>";

	&read_marc_record();

	if ($FULL) { &display_marc_record($FULL); }
	else       { &default_full_marc_record(); }

	&print_opac_info();

        &print_local_link() if ($LOCALIDX);	    
    }
    print '</DL>';
}

sub marc_display {
    $HYPERTAG = 'NONE'; 
    $LOCALTAG = 'NONE';

    print '<PRE>';

    while (<STDIN>) {
        print "\n<H3>$_</H3>\n";
        while (<>) {
            last if (length($_) == 1);
            tr/\x1F/$/;
            print '<BR><B>' . substr($_,0,3) . '</B>' . substr($_,3); 
        }
    }
    print '</PRE>';
}

sub brief_display { 
    $HYPERTAG = 'NONE'; 
    $LOCALTAG = 'NONE';

    if ($KEY_ACCESS) {
	($KEYTAG, $KEYINDEX, $KEYCMD) = eval("($KEY_ACCESS)");
    }

    print '<DL>';

    while (<STDIN>) {
        ($word, $number) = split;

        &read_marc_record();

        $link = &link_to_record(); 
       
        print "<DT><B>$link\">$word $number</A></B><DD>";

	if ($BRIEF) { &display_marc_record($BRIEF); }
        else        { &default_brief_marc_record(); } 

        print "\n";

    }
    print '</DL>';
}

sub read_marc_record {

#   Initialize

    @TAG = ();
    @VALUE = ();

    $LOCALIDX = '';
    $LOCALCNT = 0;

    while (<STDIN>) {
        chop;
        last unless ($_);

	if (!m/^\d\d\d/) {
	    push(@OPAC, $_);
	}
	else {
	    $tag = substr($_,0,3);
	    $value = substr($_,4);
	    push(@TAG,$tag);
	    push(@VALUE,$value);

	    &create_local_link($tag, $value) if ($tag =~ m/$LOCALTAG/o);
	}
    }
}

sub print_label {
    local($label,$field,$subfields,$subfsep,$endpunct,$cmd) = @_;

    $value = '';

    TAGLOOP: for ($i = 0; $i <= $#TAG; $i++) {
	if ($TAG[$i] =~ m/$field/) {
	    if ($TAG[$i] =~ m/$HYPERTAG/o) {
		#
		# A hypertexted Marc field
		#

		($hypersubfield, $index, $hcmd) = split(' ', $HYPERLINK{$TAG[$i]}, 3);

		$subfields = '.' unless ($subfields);

		foreach (split("\x1F",substr($VALUE[$i],3))) {
		    if (m/^$subfields/) {

			$sf = substr($_,0,1);

			$text = substr($_,2);
			
			if ($sf =~ m/^$hypersubfield/) {

			    $_ = &hypertext_value($text, $index, $hcmd);

			    $text = $PREF . $_ . "$ZINFO\">$text</A>" if ($_);
			}

			$value .= $subfsep if ($value);
			$value .= $text;
		    }
		}
	    }
	    else {  
		if (!$subfields) {
		    $_ = $VALUE[$i];
                    eval($cmd) if ($cmd);
		    s/^.. \x1F. *//g;
		    s/\x1F./$subfsep/g;
                    $value .= $subfsep if ($value);
		    $value .= $_;
		}
		else {
		    foreach (split("\x1F",substr($VALUE[$i],3))) {
			if (m/^$subfields/) {
                            eval($cmd) if ($cmd);
			    $value .= $subfsep if ($value);
			    $value .= substr($_,2);
			}
		    }
		}
	    }
	    $TAG[$i] = '   ';
        }
    }

    $value .= $subfsep if ($value && $PREVIOUS);
    $value .= $PREVIOUS;       

    if ($label) {
        print "$label$value$endpunct" if ($value);
        $PREVIOUS = '';
    }
    else {
        $PREVIOUS = $value;
    }
}

sub display_marc_record {
    local($fields) = shift;

    # Use the value of $fields to output each label

    foreach (split("\n", $fields)) {
	&print_label(eval("($_)"));
    }
}

sub default_full_marc_record {
    &print_label('<DT><B>Author:</B><DD> ' , '1..', '', ' ', ".\n");
    &print_label('<DT><B>Title:</B><DD> '  , '245', '', ' ', ".\n");
    &print_label('<DT><B>Abbreviated Title:</B><DD> '  , '210', '', ' ', ".\n");
    &print_label('<DT><B>Also Known As:</B><DD> '  , '211', '', ' ', ".\n");
    &print_label('<DT><B>Variant Title:</B><DD> '  , '212', '', ' ', ".\n");
    &print_label('<DT><B>Augmented Title:</B><DD> '  , '214', '', ' ', ".\n");
    &print_label('<DT><B>Collective Title:</B><DD> '  , '243', '', ' ', ".\n");
    &print_label('<DT><B>Cover/Spine Title:</B><DD> '  , '246', '[ahnpbgfde]', ' ', ".\n");
    &print_label('<DT><B>Former Title:</B><DD> '  , '247', '', '[ahnpbgfde]', ".\n");
    &print_label('<DT><B>Publisher:</B><DD> ', '260', '', ' ', ".\n");
    &print_label('<DT><B>Imprint:</B><DD> ', '262', '', ' ', ".\n");
    &print_label('<DT><B>Pages:</B><DD> '  , '300', '', ' ', ".\n");
    &print_label('<DT><B>Physical Description:</B><DD> '  , '30[15]', '', ' ', ".\n");
    &print_label('<DT><B>Series:</B><DD> ' , '4..', '', ' ', "\n");
    &print_label('<DT><B>Participant:</B><DD> '  , '511', '', ' ',"\n");
    &print_label('<DT><B>Abstract:</B><DD> '  , '520', '', ' ',"\n");
    &print_label('<DT><B>Publication Type:</B><DD> ', '567', '', ' ',"\n");
    &print_label('<DT><B>Language:</B><DD> '  , '546', '', ' ',"\n");
    &print_label('<DT><B>Contents:</B><DD> '  , '505', '', ' ',"\n");
    &print_label('<DT><B>Source:</B><DD> '  , '550', '', ' ',"\n", 's|^.. \x1F. *([^\x1F]+)|"<CITE>$1</CITE> "|ge;');
    &print_label('<DT><B>Notes:</B><DD> '  , '5..', '', '<DD>',"\n");
    &print_label('<DT><B>Subjects:</B><DD> ','6..', '[a-z]', '',".\n", '$subfsep = (/^a/) ? "<BR>" : ""');
    &print_label('<DT><B>Source:</B><DD> ','7..', '', ' ',".\n");
    &print_label('<DT><B>Series:</B><DD> ','8..', '', ' ',".\n");
    &print_label('<DT><B>ISBN:</B><DD> ', '020', '[a]', ' ', "\n");
    &print_label('<DT><B>ISSN:</B><DD> ', '022', '[a]', ' ', "\n");
    &print_label('<DT><B>Coden:</B><DD> ', '030', '[a]', ' ', "\n");
    &print_label('<DT><B>Record Number:</B><DD> ', '060', '[b]', ' ', "\n");
    &print_label('<DT><B>Standard Number:</B><DD> ', '069', '[a]', ' ', "\n");
    &print_label('<DT><B>Government Document Number:</B><DD> ', '086', '', ', ', "\n");
    &print_label('<DT><B>Issue Number:</B><DD> ', '028', '', ', ', "\n");
    &print_label('<DT><B>Techical Report Number:</B><DD> ', '26[02]', '', ', ', "\n");
    &print_label('<DT><B>Call Number:</B><DD> ', '90', '', ', ', "\n");	
}

sub default_brief_marc_record {
    &print_label('', '1..', '.', ', ', '', 's/\s+$//; $_ .= "." if (m/[^.]$/);');
    &print_label(' "', '245', '', ' ', '"');
    &print_label(' ', '550', '', ' ', '');
    &print_label(' (', '045', '.', ' ', ')', 'tr/\[\]//d; s/^\s+//;');
}

sub link_to_record {
  
    #
    # Key access uses a key to access the record directly.  First
    # see if you can match the tag which holds the key.
    #
 
    if ($KEY_ACCESS) {
	for ($i = 0; $i <= $#TAG; $i++) {
	    if ($TAG[$i] =~ m/$KEYTAG/) {

		# A key exists, alter it and trim blanks.

		$_ = &hypertext_value($VALUE[$i], $KEYINDEX, $KEYCMD);

		return $PREF . $_ . "&disnum=$number";
	    }
	}
    }

    #  
    # This is a regular link, repeat the search and set the $start to   
    # pick up the record in the appropriate postion.   
    #

    $_ = $BREF . $ENV{QUERY_STRING};
    $_ .= "&start=$number" unless (s/&start=\d+/&start=$number/);
    $_ .= '&records=1'     unless (s/&records=\d+/&records=1/);
    $_ .= '&elemname=F'    unless (s/&elemname=\w+/&elemname=F/);

    return $_;
}

sub create_local_link {
    local($tag, $value) = @_;

    local($subfields, $index, $lop, $hcmd) = split(' ', $LOCALLINK{$tag}, 4);

    $subfields = '.' unless ($subfields);

    foreach (split("\x1F",substr($value,3))) {
	if (m/^$subfields/) {

	    $_ = &hypertext_value(substr($_,2), $index, $hcmd, $LOCALCNT);

	    if ($_) {
		$LOCALIDX .= "&lop$LOCALCNT=$lop" if ($LOCALIDX);

		$LOCALIDX .= $_;

		$LOCALCNT++;
	    }
	}
    }
}

sub print_local_link {

    if (defined $LOCAL_SUBMIT) {
	print "<DT>\n<FORM METHOD=GET ACTION=\"";
	$_ = $HREF;
	s/search.CGI/searchspi.CGI/ if ($ENV{SERVER_NAME} =~ m/stanford.edu$/);
	print "$_\">\n";
	print "<INPUT NAME=QS TYPE=hidden VALUE=\"$LOCAL_PROFILE$LOCALIDX$ZINFO\">\n";
	print "<INPUT TYPE=submit VALUE=\"Search $LOCAL_NAME\">\n";
	print "</FORM>";
    }
    else {
	print "<DT><BR><IMG SRC=image/Find.gif ALIGN=MIDDLE ALT=Find><B> ";
	print 'this record in ';
	$_ = $BREF . $LOCAL_PROFILE . $LOCALIDX . $ZINFO;
	s/search.CGI/searchspi.CGI/ if ($ENV{SERVER_NAME} =~ m/stanford.edu$/);
	print "$_\">$LOCAL_NAME</A></B>\n";
    }

}

sub print_opac_info {
    $first = 1;

    foreach (@OPAC) {
	($tag, $value) = split(' ', $_, 2);

        if ($tag eq 'LocalLocation') {
            $lloc = $value;
	}
	elsif ($tag eq 'ShelvingLocation') {
	    $lshelf = '(' . $value . ')';
	}
	elsif ($tag eq 'CallNumber') {
	    if ($first) {
		print '<DT><B>Call Numbers:</B><DD>';
		$first = 0;
	    }
	    else {
                 print '<BR>';
            }
	    print "$lloc  <TAB>$value  $lshelf\n";
	}
    }
}

sub hypertext_value {

    local($value, $index, $hcmd, $indexcnt) = @_;

    $indexcnt = '0' unless ($indexcnt);

    #
    # Create a hypertext link from a Marc field
    #

    $_ = $value;
			

    # Trim leading and trailing blanks
    # This has been benchmarked to be the fastest

    s/^\s+//; s/\s+$//;

    # Perform possible command on link.
    # This can be used to filter the search text.

    eval($hcmd) if ($hcmd);

    # If nothing left then do not perform a 
    # hypertext link and just used the regular
    # text.

    if ($_) {
	# Convert special characters to ascii with % escape character. 

	s|([\x00-\x20\"\#%&/;<>?\x7F-\xFF])|sprintf("%%%02lx",unpack('C',$1))|ge; # The " balances the quote marks for the editor.
		
	$_ = '&' . $indexcnt . '_' . "$index=$_";
    }

    return $_;
}

sub setup_hypertext {

    # Setup the ending reference for hyptext links

    $_ = $ENV{QUERY_STRING};
    s/^$ZDB//;
    s/[&]*\d+_[^&]+|&lop[^&]+//g;   #  Get rid of search terms and logical operators
    s/&start=\d+/&start=1/;
    s/&records=\d+/&records=25/;
    s/&elemname=\w+/&elemname=B/;
    $ZINFO = $_;

#
#   Index the hypertext with the tag you want linked.
#

    if (defined($HYPERTEXT)) {
        foreach (split("\n", $HYPERTEXT)) {
 	   ($tag, $match, $index, $cmd) = eval('(' . $_ . ')');
           die "Error in hyptertext entry: $_\n  $@" if ($@);  
           $HYPERLINK{$tag} = join(' ',$match, $index, $cmd);
        }
    } 

    $HYPERTAG = join('|', keys %HYPERLINK);
    $HYPERTAG = 'NOTAGS' unless ($HYPERTAG);

#
#   Now check for any local link that you want.
#

    if ($LOCAL_LOOKUP && $LOCAL_PROFILE) {
        foreach (split("\n", $LOCAL_LOOKUP)) {
           ($field, $subfields, $index, $lop, $cmd) = eval('(' . $_ . ')');
           $LOCALLINK{$field} = join(' ', $subfields, $index, $lop, $cmd);
        }
    } 
	
    $LOCALTAG = join('|', keys %LOCALLINK);
    $LOCALTAG = 'NOTAGS' unless ($LOCALTAG);
}
