#!/usr/local/bin/perl
#
# This script will take a query and search or scan a Z39.50 database. 
#
# Author: Harold Finkbeiner (harold@lindy.stanford.edu)
#
# Setup the site variables
#
# Trace file for commands sent to ZCLIENT.  Enter name of file
# like '/tmp/zin' or else '' for NO tracing.  Make sure the
# file allows public write.
#
$ZCLIENT_TRACE_FILE = ''; 

do "site.pl";

# For debugging purposes only uncomment the next line.
# do "env.pl";

# Check for special QS (query string) assignment.
# This is used when using a submit form to send
# the URL across.

if ($ENV{QUERY_STRING} =~ m/^QS=/) {
    $_ = $ENV{QUERY_STRING};
    s/^QS=//;
    s/%(..)/pack("c",hex($1))/ge;
    $ENV{QUERY_STRING} = $_;
}

# If necessary add the path to include cgi-lib.pl

push(@INC, $WWWCGI);
require "cgi-lib.pl";

# Read in the variables from CGI into the associative array FORM

&ReadParse(*FORM);

# Check if there is a DB profile.

$ENV{QUERY_STRING} =~ m/^([^& ]*)/;

$ZDB = $1;

if (-f "db/$ZDB") {       # If the profile exists then extract profile info.
    do "db/$ZDB";

#   Make the associative array INDEX for the indexes.
 
    foreach (split("\n", $INDEXES)) {
	($attrib, $parms) = split(',',  $_, 2);
	$INDEX{$attrib} = $parms;
    }
}
else {
    $ZDB = '';
}

$ZDB .= '&dbname=' . $1 if ($ENV{QUERY_STRING} =~ m/&dbname=([^&]+)/);

# Create the form QUERY

$ENV{QUERY_STRING} = $FORM unless ($ENV{QUERY_STRING});

$ENV{QUERY_STRING} =~ s/&cache(=[^&])?//;

$| = 1;				# Flush after every print

print &PrintHeader;
print '<HTML>';
print '<!-- WWW to Z39.50 gateway developed by Harold Finkbeiner (harold@lindy.stanford.edu) -->';

#
# Check for Trace File.
#
if ($ZCLIENT_TRACE_FILE) {
   open(ZTRACE, ">>$ZCLIENT_TRACE_FILE") || 
       &exit_program("<HR><P>Unable to open the trace file $ZCLIENT_TRACE_FILE due to an error ($!)</P>");
}

#
# WWW FORM - Z39.50 INFORMATION 
#
# The following Z39.50 information comes from the WWW form.
# The names must be in lowercase.
#

# This specifies the internet name or address of the Z39.50 server. 

$host = $FORM{host};

# This specifies the port on which the Z39.50 client must
# connect.  By default the port number is 210.

$port = $FORM{port};

#Specify a preferred syntax you would like to use when records
#are displayed.  The syntax may be either USMARC (the default),
#or OPAC or SUTR (simple text) or INFO1 (generic record format).

$syntax = $FORM{syntax};

#The <name> must be either the name of the database to search
#the server or a the name of pre-selectable database (do SHOW SELECT
#to see a list of all pre-selectable databases).  If the <name> is
#the name of a database then include the host and/or port which
#identifies the Z39.50 server.

$dbname = $FORM{dbname};

if ($FORM{sdb}) {
   $SELECT_DB =~ tr/\n/,/;
   %select_db = eval('(' . $SELECT_DB . ')');
   $dbname = $select_db{$FORM{sdb}} if (defined($select_db{$FORM{sdb}})); 
   $ZDB .= '&dbname=' . $dbname;
   $ENV{QUERY_STRING} .=  '&dbname=' . $dbname;
}

#The element set name to be used.  The usual is "F" for a full display
#and "B" for a brief display.

$elemname = $FORM{elemname};

#The filter is name of a program used to filter the display.

$filter = ($FORM{filter}) ? "$ZFILTERS$FORM{filter}" : '';

# The starting position of the first record to display in the
# result set.

$start = ($FORM{start}) ? $FORM{start} : 1;

# This number of records to display.

$records = ($FORM{records}) ? $FORM{records} : 25;

#
# Z39.50 INFORMATION
#
# This is Z39.50 information which should not be distributed
# but which may be required.
# 

# This cause the client to send a Id Authentication to the server.
# Some server require such information.  The id maybe set in the 
# db directory as the variable $ATHORIZATION_ID or here in the script.
#
# $id = 'ACCOUNT/PASSWORD' if ($host eq 'host.site.domain');
# 

$id = $AUTHORIZATION_ID;

if ($id) {
   if ($ENV{REMOTE_ADDR} !~ m/$ZACCESS/) {
       print '<HR><P>The information you attempted to access it RESTRICTED, Sorry</P>';
       &exit_program();
   }
}

# Now parse the query to get displayable search string.

&parse_query();

# What action to perform, browse or search (the default).

$action = ($FORM{action}) ? $FORM{action} : 'SEARCH';

# Put out the header of the record.

$SHORT_NAME = $dbname unless ($SHORT_NAME);

print "<HEAD><TITLE>$ZHEADER $action [$search]</TITLE></HEAD>";
print "<BODY><H2>$ZHEADER $action $SHORT_NAME [$search]</H2>";

$cache = $FORM{cache};

if ($cache && $records == 1 && -s "/tmp/$cache" > 0) {
    close(STDIN);
    $disnum = ($FORM{disnum}) ? $FORM{disnum} : $start;

    open(STDIN, "awk '/^Record $disnum\$/,/^\$/' /tmp/$cache |");

    $elemname = 'F';

    if (!eof(STDIN)) {
       do "$filter" if ($filter);
       &exit_program();
    }
}

#
# Start the Z39.50 client and cause the output of the 
# client to be redirected to STDIN. 
#

&zexecute("$ZCLIENT -terse");

# Try to select the Z39.50 database

&zcommand("SELECT $dbname",host,port,id,syntax);

$_ = <STDIN>;
if (! m/^-OK:/) { 
    &diagnostics('The database is currently unavailable'); 
    &exit_program();
}

# Now parse the query to get displayable search string.

goto SCAN if ($action eq "BROWSE");

# Construct the find command from the query entered into
# the WWW form.

$ENV{QUERY_STRING} .= $wwwquery if ($wwwquery && $ENV{'REQUEST_METHOD'} ne "GET");

for ($op = 0; $op < $nterms; $op++) {
    $find .= ' ' . $logical[$op] if ($find);
    $find .= " $index[$op] ($term[$op])";
    $words  .= ' ' . $term[$op];
}

# Try the appropriate search on the database.

&zcommand('FIND '," $find");

$_ = <STDIN>;
if (! m/^-OK: Status (\S+), Hits (\d+),.*/) {
    &diagnostics('There is a problem with the search.');
    &exit_program();
}

$status = $1; $hits= $2;

# If the number of hits is zero then try to check for 
# spelling errors (it would be nice to suggest correct
# spellings).
    
if ($hits <= 0) {

    &zmessage();

    print "No records found$_.\n";

    $_ = &spelling_check();
    if ($_) {
	print "<PRE>Please check the following words which may be misspelled\n\n";

	print $_;
	print '</PRE>';
    }
}
else {

    &zmessage();
 
    # Check if server ignored record limit and piggy backed records

    if (m/^Record /) {
	$RECORD_HEADER_READ = $_;
        $_ = '';
    }
   
    print "<P><I><B>$hits</B> records matched your query$_</I></P><HR>\n";

    # Display the records, if only one record is found force
    # a full display to be used.

    if ($hits == 1) { $elemname = 'F' };

    # The unicorn server cannot handle element set names as part of a present equest.
    
    if ($dbname eq 'medl' || $dbname eq 'ContentsFirst') {
	&mdisplay();
    }
    else {

	# Check if you want to set up a cache. 

	if (defined($FORM{cache}) && $elemname eq 'B') {
	    $cache = 'ZGC_' . time . '_' . $$;
	    $elemname = 'F';
	}
	else {
	    $cache = '';
	}

	if (!$RECORD_HEADER_READ) {
	    if ($dbname eq 'unicorn') {
		&zcommand('MDISPLAY ',records," $start");
	    }
	    else { 
		&zcommand('MDISPLAY ',elemname,records," $start");
	    }
	}

	&zcommand('CLOSE ');
	    
	$elemname = 'B' if ($cache);
	    
	do "$filter" if ($filter);
    }
}

#
# If more are available with browse records then create a link
# to more records.
#

if (($start + $records) < $hits) {
    if ($elemname eq 'B') {
        $_ = "<A HREF=\"http://$ENV{SERVER_NAME}:$ENV{SERVER_PORT}$ENV{SCRIPT_NAME}?$ENV{QUERY_STRING}";

        $start += $records;

        $_ .= "&start=$start" unless (s/&start=\d+/&start=$start/);

	$_ .= "&records=$records" unless ($records == 25 || m/&records/);

        print "<HR>Press here to get $_\"><B>More</B></A> records.\n";
    }
}

FINISH;

&exit_program();

SCAN:

# Setup the starting reference for creating hypertext links

$HREF = "<a href=\"http://$ENV{SERVER_NAME}:$ENV{SERVER_PORT}$ENV{SCRIPT_NAME}?"; # 

$HREF .= $ZDB . '&' if ($ZDB);

# Setup the ending reference for hyptext links

$_ = $ENV{QUERY_STRING};
s/^$ZDB// if ($ZDB);
s/[&]*\d+_[^&]+|&lop[^&]+//g;	#  Get rid of search terms and logical operators
s/&start=\d+/&start=1/;    
s/&records=\d+/&records=25/; 
s/&elemname=\w+/&elemname=B/;
s/&action=\w+//;

$ZINFO = $_;

# Do the scan command for each term.

for ($op = 0; $op < $nterms; $op++) {

    # Set the name/cmd and scan index for the search.

    ($name, $attributes, $cmd, $scan) = eval("($INDEX{$index[$op]})");

    # Get the term to scan for.

    $_ = $term[$op];
    eval($cmd) if ($cmd);

    # The translation is done for SiteSearch which will not work
    # properly for uppercase values

    tr/[A-Z]/[a-z]/;

    &zcommand("SCAN NUMBER=$records $index[$op] $_");
	
    $_ = <STDIN>;
    if (! m/^-OK: Status (\S+), Returned (\d+).*/) {
	&diagnostics('There is a problem with browsing.');
	exit;
    }

    $status = $1; $hits = $2;

    if (!$name) {
	$index[$op] =~ m|(\w+)|;
	$name = $1;
    }
    
    print "<P><H3>Browse $name for <B>$term[$op]</B></H3>\n";
    print "<DL>\n";

    $hits--;

    for (; $hits > 0; $hits--) {

	$_ = <STDIN>;

	m/(.+)\s\((\d+) occurrences\)/;

	$text = $1;  $occurs = $2;

	$_ = $text;

	# Trim leading and trailing blanks
	# This has been benchmarked to be the fastest

	s/^\s+//; s/\s+$//;

	# Convert special characters to ascii with % escape character.

	s|([\x00-\x20\"\#%&/;<>?\x7F-\xFF])|sprintf("%%%02lx",unpack('C',$1)) |ge; # The " balances the quote marks for the editor.

#	if (($host eq 'rlg.stanford.edu') && ($index[$op] =~ m/^SUBJECT/)) {
#	    $scan = 'SUBJECT/STRUC:PHRASE/POS:FIRST';
#	}

	$scan = $index[$op] unless ($scan);

	print '<DT>' . $HREF . "0_$scan=$_$ZINFO\">$text</A> ($occurs occurrences)\n";
    }
    print "</DL>\n";
}
    
&zcommand('CLOSE ');

&exit_program();

exit;				# Fail save 

sub parse_query {
#
# WWW FORM - QUERY INFORMATION
#
# The query which is to be passed on to the Z39.50 client.
# Form the query from index terms which are identified as
# the names which BEGIN WITH DIGIT(S) FOLLOWED BY AN 
# UNDERSCORE (_).  The digit should reflect the ordinal
# position of index in the form starting at zero (0).  
# For example:    
# 
#  <input type=text name="0_<index0>">
#  <input type=text name="1_<index1>">
#  <input type=text name="2_<index2>">
#
# There may be logical operators between the index terms 
# which are identified "lop?" where "?" is the number of
# index term.
#
    $nterms = 0;
    foreach $indexid (sort keys(%FORM)) {
	if ($indexid =~ m/^(\d+)_(.*)/) {
            $op = $1;
	    $term[$nterms] = $FORM{$indexid};
	    if ($term[$nterms]) {
		$index[$nterms] = $2;
		$logical[$nterms] = $FORM{'lop'.$op};
		if (!$logical[$nterms]) { $logical[$nterms] = 'AND' };

		$_ = $term[$nterms];

		if (defined $INDEX{$index[$nterms]}) {
		    ($name, $cmd, $scan) = eval("($INDEX{$index[$nterms]})");
		    eval($cmd) if ($cmd);
		}
		else {
		    $index[$nterms] =~ m/(\w+)/;
		    $name = $1;
		}

		s|([\x00-\x20\"\#%&/;<>?\x7F-\xFF])|sprintf("%%%02lx",unpack('C',$1))|ge; # The " balances the quote marks for the editor.

		$wwwquery .= '&' . $nterms . '_' . $index[$nterms] . '=' . $_;

		if ($nterms > 0)  { $wwwquery .= "&lop$nterms=$logical[$nterms]"; }

                # Construct a search string suitable for display
                # to the user.

                $search .= ' ' . $logical[$nterms] . ' ' if ($search);
                $search .= $name . ' '  . $term[$nterms];

		# Construct the Query string for WWW 

	        $nterms++;
	    }
        }
    }
}

#
# This is a subroutine to create a zclient command.  There maybe any
# number of parameters.  The parameter should be a literal (with a blank)
# or else the name of a variable output as variable-name=variable-value
#

sub zcommand {

   local($name,$text,$value);

   # Create the zclient command.  If the parameter has a space 
   # in it the value is a literal string.  Otherwise it is
   # the name of a variable to be put out as:
   #       <name of variable>=<value of variable>
   # if the variable has a value

   foreach $name (@_) {
      if ($name =~ m/ /) {	#  The parameter is a literal
          $text .= $name . ' ';
      }
      else {			# The parameter is a varible
          $value = qq/${$name}/;
          $text .= "$name=$value " if ($value);
      }
   }

   # For debugging you can see what commands are sent to the 
   # Z39.50 client by setting the ZCLIENT_TRACE_FILE.  Make 
   # sure the file has rw permission (chmod a+rw <filename>).
   #
   print ZTRACE $text . $/ if ($ZCLIENT_TRACE_FILE);
   #
   # Send the command to the Z39.50 client

   print ZOUT $text . "\n";

#    
# If necessary wait for IO to appear
#
#   vec($rin,fileno(STDIN),1)=1;
#   if (select($rout=$rin, undef, undef, undef) <= 0) {
#       die "Unable to read from $ZCLIENT\n";
#   }

}

#
# This is a subroutine which takes no parameters.  It will check
# STDIN to determine if input is waiting.
#

sub zinput {
    local ($rin, $rout, $nfound, $timeleft);

    vec($rin,fileno(STDIN),1)=1;

    ($nfound, $timeleft) = select($rout=$rin, undef, undef, 1);

    if ($nfound < 0) { 
        die "Unable to wait for read from $ZCLIENT\n";
    }
    return $nfound;
}

#
# This subroutine will attempt to retrieve a message from the
# input stream.
#

sub zmessage {

    local($msgtype, $msg, $info);
    $_ = '';

    while (&zinput() > 0) {
        $_ = <STDIN>;
        ($msgtype,$msg) = split(':', $_, 2);
        $msg =~ s/^ \((\d+)\)\s+//;
        if ($msgtype eq '-DIAGNOSTIC') {
            require "diag-lib.pl";
            $info = &diagnostic_message($1, $msg);
        } else {
            $info = $msg;
        }
    }

    $_ = ($info) ? ' (' . $info . ')' : $_;
}

#
# This will execute the program and route the output to
# STDIN and creates a input stream to the program called ZOUT.
#

sub zexecute {

    local($program) = shift;

    pipe (FROM_PERL, TO_PROGRAM); # Pipes to handle the
    pipe (FROM_PROGRAM, TO_PERL); #   communictation

# create a child process

    $pid = fork;
    die "Couldn't fork: $!" unless defined $pid;

# child process becomes the program

if ($pid == 0)  {

    close FROM_PROGRAM;		# Close unused parts 
    close TO_PROGRAM;		#  of pipes

    # attach standard input/output to the pipes

    close  STDIN;
    open  (STDIN,  '<&FROM_PERL') || die ("open: $!");

    close  STDOUT;
    open  (STDOUT, '>&TO_PERL')   || die ("open: $!");

    select STDOUT; $| = 1;	# Unbuffer output

    exec "$program";		# Execute the program

    die "exec: $!";		# Should NEVER get here
}

# parent process 

    close FROM_PERL;		# Close unused parts
    close TO_PERL;		#   of pipes

# Attach files to communication pipes for easy I/O

    close STDIN;
    open(STDIN,"<&FROM_PROGRAM");
    open(ZOUT,">&TO_PROGRAM");

    select(ZOUT);  $| = 1;	# Unbuffer output
    select STDOUT;
}

sub spelling_check {

    local($misspell, $choice, $first, $rest, $spellterms);

    if ($nterms < 0) { return ''; }

    $spellterms = quotemeta(join(' ',@term));
    $spellterms =~ s/\n/ /mg;

    # Use the WEBSTER dictionary to check the spelling or else just plain spell
 
    if ($WEBSTER) {

        #  The webster dictionary must have the -s option for spell
	#  checking.   Webster will return possible choices for the
        #  correct spelling as well.

	open(SPELL, "$WEBSTER -s $spellterms |");

	while (<SPELL>) {
	    chop;
	    ($first, $rest) = split(' ', $_, 2);
	    if ($first eq '--->') {
		$misspell .=  "<B>$word:</B> $choice\n" if ($choice);
		$word = $rest;
		$choice = '';
	    } 
	    elsif ($first eq "Can't") {
		$misspell .=  "<B>$word</B>\n";
	    }
	    elsif ($first ne 'Correct' && $first ne 'Maybe') {
		$choice .= ', ' if ($choice);
		$choice .= $rest;
	    }
	    endif
	    }
	$misspell .= "<B>$word:</B> $choice\n" if ($choice);
    }
    else {
	open(SPELL, "echo '$spellterms' | spell |");

	while (<SPELL>) { $misspell .= "<B>$_</B>\n"; }

	close(SPELL);
    }
    return $misspell;
}

sub diagnostics {
    $_ = shift;

    print "<PRE>$_\n";

    while (<STDIN>) {
        print $_ if (m/^-DIAGNOSTIC:|^-ERROR:/);
    }

    close(ZOUT);
}

sub mdisplay {       
   $max = ($hits > $records) ? $records : $hits;

   while ($max > 0) {
       $rec = ($max > 10) ? 10 : $max;
       &zcommand('DISPLAY ',elemname,rec," $start"); 

       $_ = <STDIN>;
       if (! m/^-OK: Status (\d+), Received (\d+),.*/) {
           &diagnostics('There is a problem displaying the records.');
           exit;
       }

       $status = $1; $received = $2;

       $status = 0 if ($status == 2);

       if ($status > 0 || $received == 0) {
           &diagnostics('There is a problem displaying the records.');
           exit;
       }

       $max -= $received;
       $start += $received;

       do "$filter" if ($filter);
    }

    &zcommand('CLOSE ');
}

sub dump_variables {
   require "dumpvar.pl";
   
   $id = '';                   # Blank out the id so no one can see this
   $AUTHORIZATION_ID = '';

   print "<HR><XMP>";
   &dumpvar('main');
   print "</XMP><HR>";
}

sub exit_program {
   for (@_) { print $_; }
   close (ZOUT);
   close (ZTRACE) if ($ZCLIENT_TRACE_FILE);

   print $ADDRESS;
   &dump_variables() if defined $FORM{dumpvars};
   print "\n</BODY>\n</HTML>";

   exit;
}
