#!/usr/local/bin/perl
#
# This script sets up site variables for the WWW Z39.50 Gateway. 
#
# Author: Harold Finkbeiner (harold@lindy.stanford.edu)
#
# Here are some constants which must be set.  You should put everthing
# inside quotes (') and put a semicolon (;) at the end of the line.

# ZCLIENT   - The location of the zclient from Stanford.  You should be able
#             to type this command at a prompt and invoke the Stanford zclient.

$ZCLIENT  = '/main/asc/z39/zclient';

# ZFILTERS  - The directory where the filters for adding HTML formatting to
#             the Z39.50 output are stored.  If the filters are in the same
#             directory as your scripts leave this variable null

$ZFILTERS = '';

# ZACCESS   - A perl pattern to match the IP address of the remote user which
#             restricts access to authorized databases.  Here is a quick 
#             summary to pattern matching in perl.
#
#             .         Any one character except a newline. 
#             c*        Match zero or more instances of character c. 
#             c+        Match one or more instances of character c. 
#             c?        Match zero or one instance of character c. 
#             [class]   Match any of the characters in character class class. 
#             \d        Match a numeric character 
#             \.        Match a period
#             ^         Match the beginning of a line 
#             $         Match the end of a line 
#             (a|b)     Match either a or b
#             
#             Site                          Pattern
#             36.100.nnn                    ^36\.100\.
#             36.100.nnn or 36.101.nnn      ^36\.10[01]\.
#             36.100.nnn or 36.275.nnn      ^36\.(100|275)\.

$ZACCESS  = '^(36|171\.65|134\.79|198\.31\.40|131\.119\.24[456]|198\.31\.3[2345])\.';

# WWWCGI    - The location of the WWW cgi-bin scripts.

$WWWCGI   = '/main/asc/public_html';

# WEBSTER   - Webster command (with -s option) for spell checking, if set to
#             null then the spell command is used to check spelling.

$WEBSTER  = '/usr/pubsw/bin/webster';

# ADDRESS   - HTML to place at the bottom of the output.

$ADDRESS  = '<HR><ADDRESS><A HREF="mailto:harold@lindy.stanford.edu">Z39.50 at Stanford</A></ADDRESS>';

# ZHEADER   - A header message for this site.

$ZHEADER  = 'DB Connect';

# To establish an automatic link to your local catalog to a
# remote Z39.50 database set must set the following.
#
# LOCAL_PROFILE - The name of the profile for your local catalog.
# LOCAL_NAME    - The name you would like to use for your local catalog.
# LOCAL_LOOKUP  - A list of fields used to create the link to your
#                 local catalog (ie. the ISSN/ISBN fields).
# LOCAL_SUBMIT  - Use a submit button instead of a hypertext link
#                 to invoke the search of the local catalog.
#
# The entries for the LOCAL_LOOKUP are used to establish a link.
# Each entry is made of 5 strings.
#
# The first string is a pattern for the field to hypertext.  The
# most common form is use the name of the field.
#
# The second string is a pattern for the subfield to hypertext.
# The most common form is to create a list of class of matching
# subfields, ie. [abdrt].
#
# The third string is the zclient name of the index to search
# along with any special attributes.
#
# The fourth string is the logical operator (AND/OR) to use for 
# this index when joining it together with a previous lookup index.
# 
# The fifth string is optional and it contains perl commands to
# execute on the search terms being passed to the index.
#
# These global settings can be overridden by local settings in a
# profile. 

$LOCAL_SUBMIT  = '';
$LOCAL_PROFILE = 'PSOC';
$LOCAL_NAME    = 'Socrates';
$LOCAL_LOOKUP = <<'end-of-local-lookup';
'020', '[a]', 'NUMBER', 'Or', 's/ .*$//'
'022', '[a]', 'NUMBER', 'Or', 's/ .*$//'
end-of-local-lookup
