#!/usr/local/bin/perl
#
# This script read a profile and create a form to search a Z39.50 database. 
#
# Author: Harold Finkbeiner (harold@lindy.stanford.edu)
#
# Setup the site variables

do "site.pl";

# The default database to choose

$ENV{QUERY_STRING} = 'CARL' unless ($ENV{QUERY_STRING});

if ($ENV{QUERY_STRING} =~ m/&/) {
    
    # Use the CGI script to read the query string.

    push(@INC, $WWWCGI);
    require "cgi-lib.pl";
    &ReadParse(*FORM);

    # Get the DB profile.

    $ENV{QUERY_STRING} =~ m/^([^& ]*)/;
    $ZDB = $1;
}
else {
    $ZDB = $ENV{QUERY_STRING};
}

print "Content-type: text/html\n";
print "\n";

if (! -f "db/$ZDB") {

# If an invalid database name print out a message
#
# NOTE: The line invalid-database-name may not be indented.

print <<"invalid-database-name";
<HTML>
<HEAD>
<TITLE>$ZHEADER</TITLE>
</HEAD>
<BODY>
<H2>$ZHEADER</H2>
<P>The query string $ZDB is not a valid database name to search. 
Please try again with a valid database name.</P>
$ADDRESS
</BODY>
<HTML>
invalid-database-name
exit;

}

do "db/$ZDB";

# Change the script name to submit the form.
#
# The profile may contain a $ZFORM_SCRIPT variable with the
# name of the script to use, otherwise use search.CGI.

$script = $ENV{SCRIPT_NAME};
$ZFORM_SCRIPT = 'search.CGI' unless ($ZFORM_SCRIPT);
$script =~ s/zform.CGI/$ZFORM_SCRIPT/;

# Check if all indexes are shown or just some.
#
# If this is only a mini list then create a pointer to a
# full list of indexes.

$mini = (defined($FORM{fullindex}) || $INDEXES !~ /fullindex/) ? '' : "Press <A HREF=\"http://$ENV{SERVER_NAME}:$ENV{SERVER_PORT}$ENV{SCRIPT_NAME}?$ENV{QUERY_STRING}&fullindex\">here</A> for a complete set of indexes.";

print <<"start-of-form";
<HTML>
<HEAD>
<TITLE>$ZHEADER $FULL_NAME</TITLE>
</HEAD>
<BODY>
<H2>$ZHEADER $FULL_NAME</H2>
<P>
$BEFORE_TEXT
<P>Enter appropriate search terms in the boxes below. $mini 
<FORM METHOD=POST ACTION="http://$ENV{SERVER_NAME}:$ENV{SERVER_PORT}$script?$ZDB">
start-of-form

&select_database() if ($SELECT_DB);

&print_indexes(split("\n", $INDEXES));

print "<P>\n<SELECT NAME=action><OPTION>SEARCH<OPTION>BROWSE</SELECT> the database for possible matches.\n" unless defined $NO_BROWSE;

print <<"end-of-form";
<P>
<INPUT TYPE="submit" value="Send Query"> <input type=reset value="Clear Query"> Start: <input name="start" size=4 value=1> Display <SELECT NAME="records"><OPTION>25<OPTION>50<OPTION>75</SELECT>
</P>
</FORM>
<P>
$AFTER_TEXT
$ADDRESS
end-of-form
 
&dump_variables() if defined $FORM{dumpvars};

print "\n</BODY>\n</HTML>";

sub print_indexes {
    local($name, $index, $parms, $extra, $num, $under);

    $under = '_';

    for ($num = 0; @_; $num++) {
        ($index, $parms) = split(',', shift, 2);

        ($name, $types) = eval("($parms)");
       
        next if ($types =~ /ignore/);
        next if ($mini && $types =~ /fullindex/);
 
        if ($num) {
	    print "<BR>\n<SELECT NAME=lop$num><OPTION>And<OPTION>Or\n</SELECT>";
	} else {
            print "<BR>";
        }
        print " $name: <input type=text name=\"$num$under$index\" size=40 maxlength=100>\n";
    }
}

sub select_database {
   print "\nDatabase to Query <SELECT NAME=sdb>";
   foreach (split('\n', $SELECT_DB)) {
      ($name, $dbname) = eval('(' . $_ . ')');
      print "<OPTION>$name";
   }
   print "\n</SELECT>\n<BR>";
} 

sub dump_variables {
   require "dumpvar.pl";

   $AUTHORIZATION_ID = '';    # Blank this out so no one can see this.

   print "<HR><XMP>";
   &dumpvar('main');
   print "</XMP><HR>";
}
